package cn.com.qmth.examcloud.core.examwork.api.controller;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.web.helpers.page.PageInfo;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.request.InsertOrUpdateStudentReq;
import cn.com.qmth.examcloud.core.basic.api.response.InsertOrUpdateStudentResp;
import cn.com.qmth.examcloud.core.examwork.api.controller.bean.ExamStudentDomain;
import cn.com.qmth.examcloud.core.examwork.base.util.ExportUtils;
import cn.com.qmth.examcloud.core.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.core.examwork.dao.bean.ExamStudentAssembler;
import cn.com.qmth.examcloud.core.examwork.dao.bean.ExamStudentDTO;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamStudent;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamStudentInfo;
import cn.com.qmth.examcloud.core.examwork.service.impl.ExamStudentServiceImpl;
import io.swagger.annotations.ApiOperation;

/**
 * 考生服务API Created by songyue on 17/1/13.
 */
@RestController
@RequestMapping("${$rmp.ctr.examwork}/exam_student")
public class ExamStudentController extends ControllerSupport {

	@Autowired
	ExamStudentRepo examStudentRepo;

	@Autowired
	ExamStudentServiceImpl examStudentService;

	@Autowired
	ExamStudentAssembler examStudentAssembler;

	@Autowired
	ExamRepo examRepo;

	@Autowired
	StudentCloudService studentCloudService;

	@GetMapping("/query")
	public List<ExamStudent> find(
			@RequestParam(value = "student_id", required = false) Long studentId) {

		return examStudentRepo.findAll((root, query, cb) -> {

			List<Predicate> predicates = new ArrayList<Predicate>();

			if (studentId != null) {
				predicates.add(cb.equal(root.get("studentId"), studentId));
			}

			Predicate[] pre = new Predicate[predicates.size()];
			return query.where(predicates.toArray(pre)).getRestriction();
		});
	}

	/**
	 * 重构
	 *
	 * @author WANGWEI
	 * @param examCriteria
	 * @param curPage
	 * @param pageSize
	 * @return
	 */
	@ApiOperation(value = "查询考试学生带条件和分页", notes = "带条件带分页")
	@GetMapping("/all/{curPage}/{pageSize}")
	public PageInfo<ExamStudentDomain> getExamStudentList(
			@ModelAttribute ExamStudentDomain examCriteria, @PathVariable Integer curPage,
			@PathVariable Integer pageSize) {
		Specification<ExamStudent> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.equal(root.get("rootOrgId"), getRootOrgId()));

			if (null != examCriteria.getOrgId()) {
				predicates.add(cb.equal(root.get("orgId"), examCriteria.getOrgId()));
			}

			if (null != examCriteria.getExamId()) {
				predicates.add(cb.equal(root.get("exam").get("id"), examCriteria.getExamId()));
			}
			if (StringUtils.isNotEmpty(examCriteria.getStudentName())) {
				predicates.add(cb.like(root.get("name"),
						toSqlSearchPattern(examCriteria.getStudentName())));
			}
			if (StringUtils.isNotEmpty(examCriteria.getStudentCode())) {
				predicates.add(cb.like(root.get("studentCode"),
						toSqlSearchPattern(examCriteria.getStudentCode())));
			}
			if (StringUtils.isNotEmpty(examCriteria.getCourseCode())) {
				predicates.add(cb.equal(root.get("courseCode"), examCriteria.getCourseCode()));
			}
			if (StringUtils.isNotEmpty(examCriteria.getCourseLevel())) {
				predicates.add(cb.equal(root.get("courseLevel"), examCriteria.getCourseLevel()));
			}
			if (StringUtils.isNotEmpty(examCriteria.getCourseName())) {
				predicates.add(cb.like(root.get("courseName"),
						toSqlSearchPattern(examCriteria.getCourseName())));
			}
			if (!StringUtils.isEmpty(examCriteria.getExamSite())) {
				predicates.add(cb.like(root.get("examSite"),
						toSqlSearchPattern(examCriteria.getExamSite())));
			}
			if (StringUtils.isNotEmpty(examCriteria.getIdentityNumber())) {
				predicates.add(cb.like(root.get("identityNumber"),
						toSqlSearchPattern(examCriteria.getIdentityNumber())));
			}
			if (StringUtils.isNotEmpty(examCriteria.getSpecialtyName())) {
				predicates.add(cb.like(root.get("specialtyName"),
						toSqlSearchPattern(examCriteria.getSpecialtyName())));
			}
			if (StringUtils.isNotEmpty(examCriteria.getInfoCollector())) {
				predicates.add(cb.like(root.get("infoCollector"),
						toSqlSearchPattern(examCriteria.getInfoCollector())));
			}
			if (examCriteria.getFinished() != null) {
				predicates.add(cb.equal(root.get("finished"), examCriteria.getFinished()));
			}

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		PageRequest pageRequest = new PageRequest(curPage, pageSize,
				new Sort(Direction.DESC, "updateTime"));

		Page<ExamStudent> examStudents = examStudentRepo.findAll(specification, pageRequest);

		List<ExamStudentDomain> ret = Lists.newArrayList();

		for (ExamStudent cur : examStudents) {
			ExamStudentDomain bean = new ExamStudentDomain();
			bean.setId(cur.getId());
			bean.setExamId(cur.getExam().getId());
			bean.setExamName(cur.getExam().getName());
			bean.setStudentId(cur.getStudentId());
			bean.setStudentName(cur.getName());
			bean.setStudentCode(cur.getStudentCode());
			bean.setIdentityNumber(cur.getIdentityNumber());
			bean.setCourseCode(cur.getCourseCode());
			bean.setCourseName(cur.getCourseName());
			bean.setInfoCollector(cur.getInfoCollector());
			bean.setOrgId(cur.getOrgId());
			bean.setOrgCode(cur.getOrgCode());
			bean.setOrgName(cur.getOrgName());
			bean.setPaperType(cur.getPaperType());
			bean.setPhone(cur.getPhone());
			bean.setGrade(cur.getGrade());
			bean.setSpecialtyName(cur.getSpecialtyName());
			bean.setExamSite(cur.getExamSite());
			bean.setFinished(cur.getFinished());
			bean.setExamType(cur.getExam().getExamType().name());
			bean.setUpdateTime(cur.getUpdateTime());
			ret.add(bean);
		}
		return new PageInfo<ExamStudentDomain>(examStudents, ret);
	}

	@ApiOperation(value = "按ID查询考试学生", notes = "ID查询")
	@GetMapping("/{id}")
	public ExamStudent getExamStudentById(@PathVariable Long id) {
		ExamStudent es = examStudentService.findById(id);
		return es;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param request
	 * @param examStudent
	 * @return
	 */
	@Transactional
	@ApiOperation(value = "新增考试学生", notes = "新增")
	@PostMapping()
	public ExamStudentDomain addExamStudent(HttpServletRequest request,
			@RequestBody ExamStudentDomain examStudent) {
		trim(examStudent);
		Long rootOrgId = getRootOrgId();
		examStudent.setRootOrgId(rootOrgId);
		examStudent.setInfoCollector(examStudent.getInfoCollector());
		return saveExamStudent(examStudent);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examStudent
	 * @return
	 */
	@Transactional
	@ApiOperation(value = "更新考试学生", notes = "更新")
	@PutMapping()
	public ExamStudentDomain updateExamStudent(@RequestBody ExamStudentDomain examStudent) {
		trim(examStudent);
		Long rootOrgId = getRootOrgId();
		examStudent.setRootOrgId(rootOrgId);
		examStudent.setInfoCollector(examStudent.getInfoCollector());
		return saveExamStudent(examStudent);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examStudent
	 * @return
	 */
	private ExamStudentDomain saveExamStudent(ExamStudentDomain examStudent) {
		InsertOrUpdateStudentReq saveStudentReq = new InsertOrUpdateStudentReq();
		saveStudentReq.setIdentityNumber(examStudent.getIdentityNumber());
		saveStudentReq.setName(examStudent.getStudentName());
		saveStudentReq.setPhoneNumber(examStudent.getPhone());
		saveStudentReq.setOrgCode(examStudent.getOrgCode());
		saveStudentReq.setOrgId(examStudent.getOrgId());
		saveStudentReq.setOrgName(examStudent.getOrgName());
		saveStudentReq.setRemark(examStudent.getRemark());
		saveStudentReq.setRootOrgId(examStudent.getRootOrgId());
		saveStudentReq.setStudentCode(examStudent.getStudentCode());

		InsertOrUpdateStudentResp insertOrUpdateStudentResp = studentCloudService
				.insertOrUpdateStudent(saveStudentReq);

		Long studentId = insertOrUpdateStudentResp.getStudentId();

		ExamStudentInfo info = new ExamStudentInfo();
		info.setCourseCode(examStudent.getCourseCode());
		info.setCourseLevel(examStudent.getCourseLevel());
		info.setCourseName(examStudent.getCourseName());
		info.setExamId(examStudent.getExamId());
		info.setIdentityNumber(examStudent.getIdentityNumber());
		info.setPaperType(examStudent.getPaperType());
		info.setRootOrgId(examStudent.getRootOrgId());
		info.setStudentCode(examStudent.getStudentCode());
		info.setStudentName(examStudent.getStudentName());
		info.setStudentId(studentId);
		info.setInfoCollector(examStudent.getInfoCollector());
		info.setGrade(examStudent.getGrade());
		info.setExamSite(examStudent.getExamSite());
		info.setSpecialtyName(examStudent.getSpecialtyName());
		info.setRemark(examStudent.getRemark());

		ExamStudentInfo savedExamStudent = examStudentService.saveExamStudent(info);

		ExamStudentDomain ret = new ExamStudentDomain();
		ret.setId(savedExamStudent.getId());
		ret.setStudentId(studentId);
		ret.setStudentCode(savedExamStudent.getStudentCode());
		ret.setStudentName(savedExamStudent.getStudentName());
		ret.setCourseCode(savedExamStudent.getCourseCode());
		ret.setCourseName(savedExamStudent.getCourseName());
		return ret;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param ids
	 * @return
	 */
	@Transactional
	@ApiOperation(value = "按ID删除考试学生", notes = "删除")
	@DeleteMapping("/{ids}")
	public List<Long> deleteExamStudent(@PathVariable String ids) {
		List<Long> examStuIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		examStudentService.deleteExamStudent(examStuIds);
		return examStuIds;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examId
	 * @return
	 */
	@ApiOperation(value = "按考试删除考试学生", notes = "按考试删除")
	@Transactional
	@DeleteMapping("/exam/{examId}")
	public Long deleteExamStudents(@PathVariable Long examId) {
		examStudentService.deleteExamStudents(examId);
		return examId;
	}

	@ApiOperation(value = "下载导入模板", notes = "下载导入模板")
	@GetMapping("/download")
	public void importFileTemplate(HttpServletResponse response) {
		List<ExamStudentDTO> list = new ArrayList<ExamStudentDTO>();
		ExportUtils.exportEXCEL("考生导入模板", ExamStudentDTO.class, list, response);
	}

	@ApiOperation(value = "导出考试学生带条件", notes = "导出")
	@GetMapping("/export")
	public void exportExamStudent(@ModelAttribute ExamStudentDTO examCriteria,
			HttpServletResponse response) {
		List<ExamStudentDTO> list = new ArrayList<ExamStudentDTO>();
		examStudentService.getAllExamStudent(examCriteria).forEach(c -> {
			list.add(examStudentAssembler.toDTO(c));
		});
		ExportUtils.exportEXCEL("课程列表", ExamStudentDTO.class, list, response);

	}

	@ApiOperation(value = "复制考试学生", notes = "复制")
	@Transactional
	@PostMapping("/copy/{sourceExamId}/{targetExamId}")
	public void copyExamStudent(@PathVariable Long sourceExamId, @PathVariable Long targetExamId) {
		examStudentService.copyExamStudent(sourceExamId, targetExamId);
	}

}
