package cn.com.qmth.examcloud.core.examwork.api.provider;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.examwork.base.enums.ExamProperty;
import cn.com.qmth.examcloud.core.examwork.dao.ExamPropertyRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.Exam;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamPropertyEntity;
import cn.com.qmth.examcloud.core.examwork.dao.enums.ExamType;
import cn.com.qmth.examcloud.core.examwork.service.impl.ExamService;
import cn.com.qmth.examcloud.examwork.api.ExamCloudService;
import cn.com.qmth.examcloud.examwork.api.request.GetExamPropertyReq;
import cn.com.qmth.examcloud.examwork.api.request.GetExamReq;
import cn.com.qmth.examcloud.examwork.api.request.SaveExamReq;
import cn.com.qmth.examcloud.examwork.api.request.SetExamPropertyReq;
import cn.com.qmth.examcloud.examwork.api.response.GetExamPropertyResp;
import cn.com.qmth.examcloud.examwork.api.response.GetExamResp;
import cn.com.qmth.examcloud.examwork.api.response.SaveExamResp;
import cn.com.qmth.examcloud.examwork.api.response.SetExamPropertyResp;
import io.swagger.annotations.ApiOperation;

/**
 * @author chenken
 * @date 2018年5月3日 下午2:08:59
 * @company QMTH
 * @description ExamProvider.java
 */
@RestController
@RequestMapping("${$rmp.cloud.examwork}" + "exam")
public class ExamCloudServiceProvider extends ControllerSupport implements ExamCloudService {

	private static final long serialVersionUID = 6850508158980856785L;

	@Autowired
	private ExamService examService;

	@Autowired
	ExamRepo examRepo;

	@Autowired
	ExamPropertyRepo examPropertyRepo;

	@ApiOperation(value = "新增考试批次", notes = "新增")
	@PostMapping("saveExam")
	@Override
	public SaveExamResp saveExam(@RequestBody SaveExamReq req) {
		Exam exam = new Exam();
		exam.setEnable(true);
		exam.setName(req.getName());
		exam.setRemark(req.getRemark());
		exam.setDuration(req.getDuration());
		exam.setExamType(ExamType.valueOf(req.getExamType()));
		exam.setBeginTime(req.getBeginTime());
		exam.setEndTime(req.getEndTime());
		exam.setRootOrgId(req.getRootOrgId());
		exam.setCanStuDel(true);
		if (!examService.checkExamName(exam)) {
			throw new StatusException("E-002001", "考试名称已存在,请重新填写");
		}
		Exam saved = examService.insertExam(exam);

		SaveExamResp resp = new SaveExamResp();
		resp.setExamId(saved.getId());
		return resp;
	}

	@ApiOperation(value = "根据名称和机构ID查询考试", notes = "根据名称和机构ID查询考试")
	@PostMapping("getExam")
	@Override
	public GetExamResp getExam(@RequestBody GetExamReq req) {
		Long id = req.getId();
		Long rootOrgId = req.getRootOrgId();
		String name = req.getName();
		if (null == id && StringUtils.isBlank(name)) {
			throw new StatusException("E-002002", "id,name不能都为空");
		}
		if (null != id && StringUtils.isNotBlank(name)) {
			throw new StatusException("E-002003", "id,name不能都不为空");
		}
		Exam exam = null;

		if (null != id) {
			exam = examRepo.findOne(id);
		} else if (StringUtils.isNotBlank(name)) {
			if (null == rootOrgId) {
				throw new StatusException("E-002004", "rootOrgId is null");
			}
			exam = examService.findExamByNameAndRootOrgId(name, rootOrgId);
		}

		if (null == exam) {
			throw new StatusException("E-002005", "考试不存在");
		}
		GetExamResp examResp = new GetExamResp();
		examResp.setId(exam.getId());
		examResp.setName(exam.getName());
		return examResp;
	}

	@Override
	public SetExamPropertyResp setExamProperty(@RequestBody SetExamPropertyReq req) {
		Long examId = req.getExamId();
		String key = req.getKey();
		String value = req.getValue();

		ExamProperty prop = ExamProperty.valueOf(key);

		ExamPropertyEntity entity = new ExamPropertyEntity();
		entity.setExamId(examId);
		entity.setKeyId(prop.getKeyId());
		entity.setValue(value);

		ExamPropertyEntity saved = examPropertyRepo.save(entity);

		SetExamPropertyResp resp = new SetExamPropertyResp();
		resp.setPropertyId(saved.getId());
		return resp;
	}

	@Override
	public GetExamPropertyResp getExamProperty(@RequestBody GetExamPropertyReq req) {
		Long examId = req.getExamId();
		String key = req.getKey();

		ExamProperty prop = ExamProperty.valueOf(key);

		ExamPropertyEntity entity = examPropertyRepo.findByexamIdAndKeyId(examId, prop.getKeyId());

		GetExamPropertyResp resp = new GetExamPropertyResp();
		if (null != entity) {
			resp.setExisting(true);
			resp.setKey(key);
			resp.setPropertyId(entity.getId());
			resp.setValue(entity.getValue());
		} else {
			resp.setExisting(false);
		}
		return resp;
	}

}
