package cn.com.qmth.examcloud.core.examwork.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.criteria.Predicate;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.BeanCopierUtil;
import cn.com.qmth.examcloud.core.basic.api.CourseCloudService;
import cn.com.qmth.examcloud.core.basic.api.OrgCloudService;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.basic.api.bean.OrgBean;
import cn.com.qmth.examcloud.core.basic.api.bean.StudentBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetOrgReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetStudentReq;
import cn.com.qmth.examcloud.core.basic.api.request.SaveCourseReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetOrgResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetStudentResp;
import cn.com.qmth.examcloud.core.basic.api.response.SaveCourseResp;
import cn.com.qmth.examcloud.core.examwork.dao.ExamOrgTimeRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.core.examwork.dao.bean.ExamStudentAssembler;
import cn.com.qmth.examcloud.core.examwork.dao.bean.ExamStudentDTO;
import cn.com.qmth.examcloud.core.examwork.dao.entity.Exam;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamOrgTime;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamStudent;
import cn.com.qmth.examcloud.core.examwork.dao.enums.ExamType;
import cn.com.qmth.examcloud.core.examwork.service.ExamStudentService;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamStudentInfo;

/**
 * 考试学生服务类 Created by songyue on 17/1/14.
 */
@Service
public class ExamStudentServiceImpl implements ExamStudentService {

	protected static final Logger log = LoggerFactory.getLogger(ExamStudentServiceImpl.class);

	@Autowired
	ExamStudentRepo examStudentRepo;

	@Autowired
	ExamRepo examRepo;

	@Autowired
	ExamStudentAssembler examStudentAssembler;

	@Autowired
	ExamOrgTimeRepo examOrgTimeRepo;

	@Autowired
	OrgCloudService orgCloudService;

	@Autowired
	ExamStudentServiceImpl examStudentService;

	@Autowired
	StudentCloudService studentCloudService;

	@Autowired
	CourseCloudService courseCloudService;

	/**
	 * 获取所有考试学生(分页)
	 * 
	 * @param examCriteria
	 * @param pageable
	 * @return
	 * @return
	 */
	public Page<ExamStudent> getAllExamStudent(ExamStudentDTO examCriteria, Pageable pageable) {
		Specification<ExamStudent> specification = getPageSpecification(examCriteria);
		Page<ExamStudent> examStudents = examStudentRepo.findAll(specification, pageable);
		processExamStudent(examStudents.getContent());
		return examStudents;
	}

	/**
	 * 获取所有考试学生
	 * 
	 * @param examCriteria
	 * @return
	 * @return
	 */
	public List<ExamStudent> getAllExamStudent(ExamStudentDTO examCriteria) {
		Specification<ExamStudent> specification = getSpecification(examCriteria);
		List<ExamStudent> examStudents = examStudentRepo.findAll(specification,
				new Sort(Direction.DESC, "id"));
		processExamStudent(examStudents);
		return examStudents;
	}

	/**
	 * 处理考生集合(设置学习中心考试时间)
	 * 
	 * @param examStudents
	 */
	private void processExamStudent(List<ExamStudent> examStudents) {
		for (ExamStudent examStudent : examStudents) {
			// 设置学习中心考试时间
			setExamOrgTime(examStudent);
		}
	}

	/**
	 * 设置学习中心考试时间
	 * 
	 * @param examStudent
	 */
	private void setExamOrgTime(ExamStudent examStudent) {
		Exam exam = examStudent.getExam();
		if (exam != null) {
			if (exam.getExamType() == ExamType.OFFLINE) {
				Long examId = exam.getId();
				Long orgId = examStudent.getOrgId();
				ExamOrgTime examOrgTime = examOrgTimeRepo.findFirstByExamIdAndOrgId(examId, orgId);
				if (examOrgTime != null) {
					String examBeginTime = String.valueOf(examOrgTime.getBeginTime().getTime());
					String examEndTime = String.valueOf(examOrgTime.getEndTime().getTime());
					examStudent.setExamBeginTime(examBeginTime);
					examStudent.setExamEndTime(examEndTime);
					examStudent.setCanUpload(getCanUpload(examOrgTime));
				}
			} else {
				String examBeginTime = String.valueOf(exam.getBeginTime().getTime());
				String examEndTime = String.valueOf(exam.getEndTime().getTime());
				examStudent.setExamBeginTime(examBeginTime);
				examStudent.setExamEndTime(examEndTime);
			}
		}
	}

	private Boolean getCanUpload(ExamOrgTime examOrgTime) {
		if (examOrgTime.getCanUpload() == null) {
			Date now = new Date();
			if (now.after(examOrgTime.getEndTime())) {
				return false;
			} else {
				return true;
			}
		} else {
			return examOrgTime.getCanUpload();
		}
	}

	public ExamStudent findById(Long id) {
		ExamStudent examStudent = examStudentRepo.findOne(id);
		setExamOrgTime(examStudent);
		return examStudent;
	}

	/**
	 * 按考试批次获取学生
	 * 
	 * @param examId
	 * @return
	 * @return
	 */
	public Page<ExamStudent> getExamStudentByExamId(Long examId, Pageable pageable) {
		return examStudentRepo.findByExamId(examId, pageable);
	}

	/**
	 * 生成查询条件
	 * 
	 * @param examCriteria
	 * @return
	 */
	private Specification<ExamStudent> getSpecification(ExamStudentDTO examCriteria) {
		Specification<ExamStudent> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			if (null != examCriteria.getStudentId()) {
				predicates.add(cb.equal(root.get("studentId"), examCriteria.getStudentId()));
			}
			if (null != examCriteria.getRootOrgId()) {
				predicates.add(cb.equal(root.get("rootOrgId"), examCriteria.getRootOrgId()));
			}
			if (null != examCriteria.getOrgId()) {
				predicates.add(cb.equal(root.get("orgId"), examCriteria.getOrgId()));
			}
			if (!StringUtils.isEmpty(examCriteria.getOrgName())) {
				predicates.add(cb.like(root.get("orgName"), "%" + examCriteria.getOrgName() + "%"));
			}
			if (null != examCriteria.getExamId()) {
				predicates.add(cb.equal(root.get("exam").get("id"), examCriteria.getExamId()));
			}
			if (!StringUtils.isEmpty(examCriteria.getName())) {
				predicates.add(cb.like(root.get("name"), "%" + examCriteria.getName() + "%"));
			}
			if (!StringUtils.isEmpty(examCriteria.getStudentCode())) {
				predicates.add(cb.like(root.get("studentCode"),
						"%" + examCriteria.getStudentCode() + "%"));
			}
			if (!StringUtils.isEmpty(examCriteria.getCourseCode())) {
				predicates.add(cb.equal(root.get("courseCode"), examCriteria.getCourseCode()));
			}
			if (!StringUtils.isEmpty(examCriteria.getCourseLevel())) {
				predicates.add(cb.equal(root.get("courseLevel"), examCriteria.getCourseLevel()));
			}
			if (!StringUtils.isEmpty(examCriteria.getCourseName())) {
				predicates.add(
						cb.like(root.get("courseName"), "%" + examCriteria.getCourseName() + "%"));
			}
			if (null != examCriteria.getFinished()) {
				predicates.add(cb.equal(root.get("finished"), examCriteria.getFinished()));
			}
			if (!StringUtils.isEmpty(examCriteria.getExamSite())) {
				predicates
						.add(cb.like(root.get("examSite"), "%" + examCriteria.getExamSite() + "%"));
			}
			if (!StringUtils.isEmpty(examCriteria.getIdentityNumber())) {
				predicates.add(
						cb.equal(root.get("identityNumber"), examCriteria.getIdentityNumber()));
			}
			if (!StringUtils.isEmpty(examCriteria.getSpecialtyName())) {
				predicates.add(cb.like(root.get("specialtyName"),
						"%" + examCriteria.getSpecialtyName() + "%"));
			}
			if (!StringUtils.isEmpty(examCriteria.getPaperType())) {
				predicates.add(cb.equal(root.get("paperType"), examCriteria.getPaperType()));
			}
			if (null != examCriteria.getDegree()) {
				predicates.add(cb.equal(root.get("degree"), examCriteria.getDegree()));
			}
			if (null != examCriteria.getRepair()) {
				predicates.add(cb.equal(root.get("repair"), examCriteria.getRepair()));
			}
			if (null != examCriteria.getGraduated()) {
				predicates.add(cb.equal(root.get("graduated"), examCriteria.getGraduated()));
			}
			if (!StringUtils.isEmpty(examCriteria.getGrade())) {
				predicates.add(cb.like(root.get("grade"), "%" + examCriteria.getGrade() + "%"));
			}
			if (!StringUtils.isEmpty(examCriteria.getExamNumber())) {
				predicates.add(
						cb.like(root.get("examNumber"), "%" + examCriteria.getExamNumber() + "%"));
			}
			if (!StringUtils.isEmpty(examCriteria.getInfoCollector())) {
				predicates.add(cb.like(root.get("infoCollector"),
						"%" + examCriteria.getInfoCollector() + "%"));
			}
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};
		return specification;
	}

	/**
	 * 生成查询条件
	 * 
	 * @param examCriteria
	 * @return
	 */
	private Specification<ExamStudent> getPageSpecification(ExamStudentDTO examCriteria) {
		Specification<ExamStudent> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			if (null != examCriteria.getStudentId()) {
				predicates.add(cb.equal(root.get("studentId"), examCriteria.getStudentId()));
			}
			if (null != examCriteria.getRootOrgId()) {
				predicates.add(cb.equal(root.get("rootOrgId"), examCriteria.getRootOrgId()));
			}
			if (null != examCriteria.getOrgId()) {
				predicates.add(cb.equal(root.get("orgId"), examCriteria.getOrgId()));
			}
			if (!StringUtils.isEmpty(examCriteria.getOrgName())) {
				predicates.add(cb.like(root.get("orgName"), "%" + examCriteria.getOrgName() + "%"));
			}
			if (null != examCriteria.getExamId()) {
				predicates.add(cb.equal(root.get("exam").get("id"), examCriteria.getExamId()));
			}
			if (!StringUtils.isEmpty(examCriteria.getName())) {
				predicates.add(cb.like(root.get("name"), "%" + examCriteria.getName() + "%"));
			}
			if (!StringUtils.isEmpty(examCriteria.getStudentCode())) {
				predicates.add(cb.like(root.get("studentCode"),
						"%" + examCriteria.getStudentCode() + "%"));
			}
			if (!StringUtils.isEmpty(examCriteria.getCourseCode())) {
				predicates.add(cb.equal(root.get("courseCode"), examCriteria.getCourseCode()));
			}
			if (!StringUtils.isEmpty(examCriteria.getCourseLevel())) {
				predicates.add(cb.equal(root.get("courseLevel"), examCriteria.getCourseLevel()));
			}
			if (!StringUtils.isEmpty(examCriteria.getCourseName())) {
				predicates.add(
						cb.like(root.get("courseName"), "%" + examCriteria.getCourseName() + "%"));
			}
			if (null != examCriteria.getFinished()) {
				predicates.add(cb.equal(root.get("finished"), examCriteria.getFinished()));
			}

			if (!StringUtils.isEmpty(examCriteria.getExamSite())) {
				predicates
						.add(cb.like(root.get("examSite"), "%" + examCriteria.getExamSite() + "%"));
			}
			if (!StringUtils.isEmpty(examCriteria.getIdentityNumber())) {
				predicates.add(cb.like(root.get("identityNumber"),
						"%" + examCriteria.getIdentityNumber() + "%"));
			}
			if (!StringUtils.isEmpty(examCriteria.getSpecialtyName())) {
				predicates.add(cb.like(root.get("specialtyName"),
						"%" + examCriteria.getSpecialtyName() + "%"));
			}
			if (!StringUtils.isEmpty(examCriteria.getPaperType())) {
				predicates.add(cb.equal(root.get("paperType"), examCriteria.getPaperType()));
			}
			if (null != examCriteria.getDegree()) {
				predicates.add(cb.equal(root.get("degree"), examCriteria.getDegree()));
			}
			if (null != examCriteria.getRepair()) {
				predicates.add(cb.equal(root.get("repair"), examCriteria.getRepair()));
			}
			if (null != examCriteria.getGraduated()) {
				predicates.add(cb.equal(root.get("graduated"), examCriteria.getGraduated()));
			}
			if (!StringUtils.isEmpty(examCriteria.getGrade())) {
				predicates.add(cb.like(root.get("grade"), "%" + examCriteria.getGrade() + "%"));
			}
			if (!StringUtils.isEmpty(examCriteria.getExamNumber())) {
				predicates.add(
						cb.like(root.get("examNumber"), "%" + examCriteria.getExamNumber() + "%"));
			}
			if (!StringUtils.isEmpty(examCriteria.getInfoCollector())) {
				predicates.add(cb.like(root.get("infoCollector"),
						"%" + examCriteria.getInfoCollector() + "%"));
			}
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};
		return specification;
	}

	/**
	 * 复制考生
	 * 
	 * @param sourceExamId
	 * @param targetExamId
	 */
	public void copyExamStudent(Long sourceExamId, Long targetExamId) {
		List<ExamStudent> sourceExamStudents = examStudentRepo.findByExamId(sourceExamId);
		List<ExamStudent> targetExamStudents = new ArrayList<ExamStudent>();
		Exam targetExam = examRepo.findOne(targetExamId);
		sourceExamStudents.stream().forEach(examStudent -> {
			if (examStudentRepo.checkExamStu(targetExam.getId(), examStudent.getRootOrgId(),
					examStudent.getIdentityNumber(), examStudent.getCourseCode()) == 0) {
				ExamStudent tempStudent = BeanCopierUtil.copyProperties(examStudent,
						ExamStudent.class);
				tempStudent.setId(null);
				tempStudent.setFinished(false);
				tempStudent.setGraduated(false);
				tempStudent.setDegree(false);
				tempStudent.setRepair(false);
				tempStudent.setCanUpload(false);
				tempStudent.setIsReexamine(false);
				tempStudent.setNormalExamTimes(0);
				tempStudent.setIsReexamine(null);
				tempStudent.setReexamineType(null);
				tempStudent.setReexamineDetail(null);
				tempStudent.setReexamineCompleted(null);
				tempStudent.setExam(targetExam);
				targetExamStudents.add(tempStudent);
			}
		});
		examStudentRepo.save(targetExamStudents);
		sourceExamStudents.clear();
		targetExamStudents.clear();
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param ids
	 */
	public void deleteExamStudent(List<Long> ids) {
		List<ExamStudent> examStudents = examStudentRepo.findByIdIn(ids);
		for (ExamStudent examStudent : examStudents) {
			if (examStudent.getExam().getCanStuDel()) {
				examStudentRepo.delete(examStudent);
			} else {
				throw new StatusException("E-150112", examStudent.getName() + "已开始考试,不能删除");
			}
		}
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examId
	 */
	public void deleteExamStudents(Long examId) {
		Exam exam = examRepo.findOne(examId);
		if (exam.getCanStuDel()) {
			examStudentRepo.deleteByExamId(examId);
		} else {
			throw new StatusException("E-150113", "该考试已开始,不能删除");
		}
	}

	/*
	 * 实现
	 *
	 * @author WANGWEI
	 * 
	 * @see cn.com.qmth.examcloud.core.examwork.service.ExamStudentService#
	 * saveExamStudent(cn.com.qmth.examcloud.core.examwork.service.bean.
	 * ExamStudentInfo)
	 */
	@Override
	public ExamStudentInfo saveExamStudent(ExamStudentInfo examStudentInfo) {
		String paperType = examStudentInfo.getPaperType();
		if (StringUtils.isBlank(paperType)) {
			paperType = "X";
		} else if (!paperType.matches("[A-Z]")) {
			throw new StatusException("E-100020", "试卷类型必须为一个大写的英文字母");
		}

		Long rootOrgId = examStudentInfo.getRootOrgId();
		GetOrgReq getOrgReq = new GetOrgReq();
		getOrgReq.setOrgId(rootOrgId);
		GetOrgResp getOrgResp = orgCloudService.getOrg(getOrgReq);
		OrgBean rootOrg = getOrgResp.getOrg();
		if (null != rootOrg.getParentId()) {
			throw new StatusException("E-100001", "rootOrgId is wrong");
		}

		Exam exam = null;
		Long examId = examStudentInfo.getExamId();
		if (null == examId) {
			String examName = examStudentInfo.getExamName();
			if (StringUtils.isBlank(examName)) {
				throw new StatusException("E-100002", "examId & examName are both blank");
			}
			List<Exam> examList = examRepo.findByNameAndRootOrgId(examName, rootOrgId);
			if (CollectionUtils.isEmpty(examList)) {
				throw new StatusException("E-100005", "考试不存在");
			}
			if (1 < examList.size()) {
				throw new StatusException("E-100005", "考试名称对应多个考试");
			}
			exam = examList.get(0);
		} else {
			exam = examRepo.findOne(examId);
		}

		String studentCode = examStudentInfo.getStudentCode();
		String identityNumber = examStudentInfo.getIdentityNumber();

		if (StringUtils.isBlank(identityNumber)) {
			throw new StatusException("E-100003", "identityNumber is null");
		}
		GetStudentReq getStudentReq = new GetStudentReq();
		getStudentReq.setRootOrgId(rootOrgId);
		getStudentReq.setStudentCode(studentCode);
		getStudentReq.setIdentityNumber(identityNumber);
		GetStudentResp getStudentResp = studentCloudService.getStudent(getStudentReq);

		StudentBean studentInfo = getStudentResp.getStudentInfo();

		String courseCode = examStudentInfo.getCourseCode();
		String courseName = examStudentInfo.getCourseName();
		String courseLevel = examStudentInfo.getCourseLevel();

		SaveCourseReq saveCourseReq = new SaveCourseReq();
		saveCourseReq.setCourseId(examStudentInfo.getCourseId());
		saveCourseReq.setCourseCode(courseCode);
		saveCourseReq.setCourseName(courseName);
		saveCourseReq.setCourseLevel(courseLevel);
		saveCourseReq.setRootOrgId(rootOrgId);

		SaveCourseResp saveCourseResp = courseCloudService.saveCourse(saveCourseReq);
		CourseBean courseBean = saveCourseResp.getCourseBean();

		List<ExamStudent> examStudentList = examStudentRepo
				.findByExamAndStudentIdAndCourseCodeOrderByUpdateTimeDesc(exam, studentInfo.getId(),
						courseBean.getCode());

		ExamStudent examStudent = null;

		if (1 < examStudentList.size()) {
			// 数据需要修正
			throw new StatusException("E-100008", "考生数据重复. studentId: " + studentInfo.getId());
		} else if (1 == examStudentList.size()) {
			examStudent = examStudentList.get(0);
		} else {
			examStudent = new ExamStudent();
			examStudent.setNormalExamTimes(0);
			examStudent.setFinished(false);
			examStudent.setIsReexamine(false);
			examStudent.setCanUpload(false);
		}
		examStudent.setInfoCollector(examStudentInfo.getInfoCollector());
		examStudent.setName(studentInfo.getName());
		examStudent.setRootOrgId(rootOrgId);

		examStudent.setCourseId(courseBean.getId());
		examStudent.setCourseCode(courseBean.getCode());
		examStudent.setCourseName(courseBean.getName());
		examStudent.setCourseLevel(courseBean.getLevel());

		examStudent.setExam(exam);
		examStudent.setIdentityNumber(studentInfo.getIdentityNumber());
		examStudent.setStudentCode(studentInfo.getStudentCode());
		examStudent.setPaperType(paperType);
		examStudent.setStudentId(studentInfo.getId());
		examStudent.setOrgId(studentInfo.getOrgId());
		examStudent.setOrgName(studentInfo.getOrgName());
		examStudent.setOrgCode(studentInfo.getOrgCode());
		examStudent.setPhone(studentInfo.getPhoneNumber());
		examStudent.setGrade(examStudentInfo.getGrade());
		examStudent.setSpecialtyName(examStudentInfo.getSpecialtyName());
		examStudent.setExamSite(examStudentInfo.getExamSite());
		examStudent.setRemark(examStudentInfo.getRemark());

		ExamStudent saved = examStudentRepo.save(examStudent);

		ExamStudentInfo ret = new ExamStudentInfo();
		ret.setId(saved.getId());
		ret.setCourseCode(saved.getCourseCode());
		ret.setCourseLevel(saved.getCourseLevel());
		ret.setCourseName(saved.getCourseName());
		ret.setExamId(saved.getExam().getId());
		ret.setExamName(saved.getExam().getName());
		ret.setIdentityNumber(saved.getIdentityNumber());
		ret.setStudentCode(saved.getStudentCode());
		ret.setPaperType(saved.getPaperType());
		ret.setRootOrgId(saved.getRootOrgId());
		ret.setStudentName(studentInfo.getName());

		return ret;
	}
}
